/** @class */
jscape.ServiceDeniedException = Class.extend(
    /** @lends {jscape.ServiceDeniedException.prototype} */
{
    /** @constructor */
    initialize: function(descriptor) {
        this.descriptor = descriptor;
    }
});

/** @class */
jscape.DomainDescriptor = Class.extend(
    /** @lends {jscape.DomainDescriptor.prototype} */
{
    /** @constructor */
    initialize: function(name, description, descriptors, configurations) {
        this.name = name;
        this.description = description;
        this.descriptors = descriptors || [];
        this.configurations = configurations || [];
    },

    toJSON: function() {
        return {
            "@class": "com.jscape.inet.mft.management.descriptors.DomainDescriptor",
            name: this.name,
            description: this.description,
            serviceDescriptors: this.descriptors,
            configurations: this.configurations
        }
    }
});
jscape.DomainDescriptor.fromJSON = function(data) {
    return new jscape.DomainDescriptor(data.name, data.description, data.serviceDescriptors, data.configurations);
};
jscape.DomainDescriptor.template = function(name, description, serverParameters) {
    var defaults = [
        {"@class": "com.jscape.inet.mft.management.descriptors.StatisticsServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.AccountTemplateServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.AccountAuthenticationServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.EventServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.ReportServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.ResourceServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.IPAccessServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.PasswordServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.management.descriptors.DirectoryMonitorServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.DropZoneServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.WebThemeServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.TradingPartnerServiceDescriptor"},
        {"@class": "com.jscape.inet.mft.AddressBookServiceDescriptor"}
    ];
    for (var i = defaults.length - 1; i--;) {
        if (!serverParameters.allowsDescriptor(defaults[i]["@class"])) {
            defaults.splice(i, 1);
        }
    }
    return new jscape.DomainDescriptor(name, description, defaults);
};

/** @class */
jscape.NewDomainModule = Class.extend(
    /** @lends {jscape.NewDomainModule.prototype} */
{
    dialog: null,
    serverParameters: null,
    managementService: null,

    /** @constructor */
    initialize: function(dialog) {
        this.dialog = dialog;
    },

    start: function(serverParameters, managementService) {
        this.serverParameters = serverParameters;
        this.managementService = managementService;
    },

    reset: function() {
    },

    valid: function() {
        return true;
    }
});

/** @class */
jscape.NewDomainController = Class.extend(
    /** @lends {jscape.NewDomainController.prototype} */
{
    dialog: null,
    modules: null,
    serverParameters: null,
    currentModuleIndex: 0,

    /** @constructor */
    initialize: function(resources) {
        this.dialog = new jscape.AddDomainDialog(resources);

        this.nameController = new jscape.DomainNameController(this.dialog, resources);
        this.serviceController = new jscape.DomainServiceController(this.dialog, resources);
        this.datastoreServiceController = new jscape.DomainDatastoreServiceController(this.dialog, resources);
        this.logServiceController = new jscape.DomainLogServiceController(this.dialog, resources);
        this.modules = [
            this.nameController,
            this.serviceController,
            this.datastoreServiceController,
            this.logServiceController
        ];
    },

    start: function(listener, managementService, serverParameters) {
        this.listener = listener || jscape.NewDomainController.LISTENER;
        this.managementService = managementService;
        this.serverParameters = serverParameters;

        this._initModules();
        this.dialog.show(this);
    },

    onPrevious: function(dialog) {
        this.currentModuleIndex--;
        return true;
    },

    onNext: function(dialog) {
        var m = this.modules[this.currentModuleIndex];
        if (m.valid()) {
            this._startModule(++this.currentModuleIndex);
            return true;
        }
        return false;
    },

    onSubmit: function(dialog) {
        try {
            this._validate();
            var descriptor = this._createDescriptor();
            this.managementService.storeDomain(descriptor).done($.proxy(function() {
                dialog.hide();
                this.managementService.startDomain(descriptor.name);
                this.serverParameters = null;
                this.listener.onComplete(descriptor);
            }, this));
        } catch (e) {
            if (e instanceof jscape.ServiceDeniedException) {
                var license = this.serverParameters.license.type;
                this.dialog.showServiceDeniedMessage(e.descriptor, license);
            }
        } finally {
            this.currentModuleIndex = 0;
        }
    },

    onCancel: function(dialog) {
        dialog.hide();
        this.currentModuleIndex = 0;
        this.listener.onCancel();
    },

    _startModule: function(index) {
        this.modules[index].start(this.serverParameters, this.managementService);
    },

    _initModules: function() {
        for (var i = 0; i < this.modules.length; i++) {
            this.modules[i].reset();
        }

        this.managementService.getDomainNames().done($.proxy(function(names) {
            this.nameController.setExistingDomains(names);
        }, this));
    },

    _validate: function() {
        this._assertDescriptorAllowed(this.serviceController.getDescriptor());
        this._assertDescriptorAllowed(this.datastoreServiceController.getDescriptor());
        this._assertDescriptorAllowed(this.logServiceController.getDescriptor());
    },

    _assertDescriptorAllowed: function(descriptor) {
        var theClass = descriptor && descriptor.toJSON()["@class"];
        if (!this.serverParameters.allowsDescriptor(theClass)) {
            throw new jscape.ServiceDeniedException(theClass);
        }
    },

    _createDescriptor: function() {
        var descriptor = jscape.DomainDescriptor.template(this.nameController.getDomainName(),
                                                          this.nameController.getDomainDescription(),
                                                          this.serverParameters);
        descriptor.descriptors = descriptor.descriptors.concat([
            this.serviceController.getDescriptor(),
            this.datastoreServiceController.getDescriptor(),
            this.logServiceController.getDescriptor()
        ]);
        return descriptor;
    }
});
jscape.NewDomainController.LISTENER = {
    onComplete: $.noop(),
    onCancel: $.noop()
};

/** @class */
jscape.AddDomainDialog = jscape.WizardDialog.extend(
    /** @lends {jscape.AddDomainDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this._super($("#add-domain-wizard"), resources);
    },

    addView: function(view) {
        this.views.push(view);
    },

    showServiceDeniedMessage: function(descriptor, license) {
        $.messager.alert(this.resources.getString("DIALOG_TITLE_ERROR"),
                         this.resources.getString("ADD_DOMAIN_ERROR_SERVICE_DENIED", {0: descriptor, 1: license}),
                         "error");
    },

    _valid: function() {
        for (var i = 0; i < this.views.length; i++) {
            if (!this.views[i].valid()) {
                return false
            }
        }
        return true;
    },

    /** @override */
    _initComponents: function() {
    },

    /** @override */
    _setViewVisible: function(view, visible) {
        if (visible) {
            view.asElement().show();
        } else {
            view.asElement().hide();
        }
    }
});

/** @class */
jscape.DomainNameController = jscape.NewDomainModule.extend(
    /** @lends {jscape.DomainNameController.prototype} */
{
    existingDomains: [],

    /** @constructor */
    initialize: function(dialog, resources) {
        this._super(dialog);

        this.view = new jscape.DomainNameView(resources);
        this.dialog.addView(this.view);

        this._initValidators(resources);
    },

    /** @override */
    start: function(serverParameters, managementService) {
        this._super(serverParameters, managementService);

        this.dialog.select(this.view);
        this.view.show(this);
    },

    /** @override */
    reset: function() {
        this.view.setName("");
        this.view.setDescription("");
    },

    /** @override */
    valid: function() {
        return this.view.valid();
    },

    setExistingDomains: function(values) {
        this.existingDomains = values || [];
    },

    getDomainName: function() {
        return this.view.getName();
    },

    getDomainDescription: function() {
        return this.view.getDescription();
    },

    _initValidators: function(resources) {
        $.extend($.fn.validatebox.defaults.rules, {
            'newdomain_name_nonempty': {
                validator: function(value) { return value && value.trim().length != 0; },
                message: resources.getString("ADD_DOMAIN_ERROR_BAD_NAME")
            },
            'newdomain_name_notexists': {
                validator: $.proxy(this._validatorDomainExists, this),
                message: resources.getString("ADD_DOMAIN_ERROR_NAME_EXISTS")
            }
        });
    },

    _validatorDomainExists: function() {
        return $.inArray(this.view.getName(), this.existingDomains) == -1;
    }
});

/** @class */
jscape.DomainNameView = Class.extend(
    /** @lends {jscape.DomainNameView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("#add-domain-step1");
        this.name = $("#domain-name");
        this.description = $("#domain-description");

        this._init();
    },

    asElement: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener || jscape.DomainNameView.LISTENER;
        this._initState();
    },

    valid: function() {
        return this.name.validatebox("isValid");
    },

    getName: function() {
        return this.name.val();
    },

    setName: function(value) {
        this.name.val(value);
    },

    getDescription: function() {
        return this.description.val();
    },

    setDescription: function(value) {
        this.description.val(value);
    },

    _init: function() {
        this.name.validatebox({
            required: true,
            validType: ["newdomain_name_nonempty", "newdomain_name_notexists"],
            missingMessage: this.resources.getString("ADD_DOMAIN_ERROR_BAD_NAME")
        });
    },

    _initState: function() {
        this.fieldset.form("validate");
    }
});
/** @const */
jscape.DomainNameView.LISTENER = {};

/** @class */
jscape.DomainServiceController = jscape.NewDomainModule.extend(
    /** @lends {jscape.DomainServiceController.prototype} */
{
    controllers: [],
    privateKeys: [],
    descriptor: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this._super(dialog);

        var ftpServiceView = new jscape.FtpServiceView(resources);
        var sftpServiceView = new jscape.SftpServiceView(resources);
        var aftpServiceView = new jscape.AftpServiceView(resources);
        var httpServiceView = new jscape.HttpServiceView(resources);
        var webDavServiceView = new jscape.WebDavServiceView(resources);
        var as2ServiceView = new jscape.AS2ServiceView(resources);
        this.controllers = [
                new jscape.FtpServiceController(ftpServiceView),
                new jscape.SftpServiceController(sftpServiceView),
                new jscape.AftpServiceController(aftpServiceView),
                new jscape.HttpServiceController(httpServiceView),
                new jscape.WebDavServiceController(webDavServiceView),
                new jscape.AS2ServiceController(as2ServiceView)
        ];
        this.view = new jscape.DomainServiceView([ftpServiceView ,
                                                  sftpServiceView,
                                                  aftpServiceView,
                                                  httpServiceView,
                                                  webDavServiceView,
                                                  as2ServiceView],
                                                 resources);
        this.dialog.addView(this.view);
    },

    /** @override */
    start: function(serverParameters, managementService) {
        this._super(serverParameters, managementService);

        this.dialog.select(this.view);

        this.managementService.getServerKeys().done($.proxy(function(keys) {
            this._initPrivateKeys(keys);
            this._startServiceControllers();
            this.view.show(this);
        }, this));
    },

    /** @override */
    reset: function() {
        this.view.reset();
    },

    /** @override */
    valid: function() {
        return this.view.valid();
    },

    getDescriptor: function() {
        this.view.apply();
        return this.descriptor;
    },

    onCompleted: function(descriptor, controller) {
        this.descriptor = descriptor;
    },

    onBroken: function(controller) {
        //ok
    },

    _startServiceControllers: function() {
        for (var i = 0; i < this.controllers.length; i++) {
            this.controllers[i].start(this, this.descriptor, this.serverParameters, this.privateKeys, this.managementService);
        }
    },

    _initPrivateKeys: function (keys) {
        this.privateKeys = new Array(keys.length);

        for (var i = 0; i < keys.length; i++) {
            this.privateKeys[i] = keys[i].alias;
        }
    }
});

/** @class */
jscape.DomainServiceView = Class.extend(
    /** @lends {jscape.DomainServiceView.prototype} */
{
    /** @constructor */
    initialize: function(views, resources) {
        this.views = views || [];
        this.resources = resources;

        this.fieldset = $("#add-domain-step2");
        this.type = $("#domain-service");

        this._init();
    },

    asElement: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener;
        this._initState();
    },

    reset: function() {
        this.type.combobox("setValue", this.type.combobox("getData")[0].value);
    },

    valid: function() {
        return this._getSelectedView().valid();
    },

    apply: function() {
        this._getSelectedView().applyChanges();
    },

    _init: function() {
        this.type.combobox({onSelect: $.proxy(this._onChange$Type, this), editable: false});
    },

    _initState: function() {
        this._adjustServiceType();
    },

    _onChange$Type: function() {
        this._adjustServiceType();
    },

    _adjustServiceType: function() {
        var selected = this.type.combobox("getValue");
        var types = this.type.combobox("getData");

        for (var i = 0; i < types.length; i++) {
            var type = types[i].value;
            var view = this._serviceViewFor(type);
            if (type == selected) {
                view.show();
            } else {
                view.hide();
            }
        }
    },

    _getSelectedView: function() {
        return this._serviceViewFor(this.type.combobox("getValue"));
    },

    _serviceViewFor: function(type) {
        return this.views[type];
    }
});

/** @class */
jscape.DomainDatastoreServiceController = jscape.NewDomainModule.extend(
    /** @lends {jscape.DomainDatastoreServiceController.prototype} */
{
    controllers: [],

    /** @constructor */
    initialize: function(dialog, resources) {
        this._super(dialog);

        var fileRecordView = new jscape.FileRecordDatastoreServiceView(resources);
        var fileView = new jscape.FileDatastoreServiceView(resources);
        var dbView = new jscape.DbDatastoreServiceView(resources);
        this.controllers = [
                new jscape.FileRecordDatastoreServiceController(fileRecordView),
                new jscape.FileDatastoreServiceController(fileView),
                new jscape.DbDatastoreServiceController(dbView)
        ];
        this.view = new jscape.DomainDatastoreServiceView([fileRecordView, fileView, dbView], resources);

        this.dialog.addView(this.view);
    },

    /** @override */
    start: function(serverParameters, managementService) {
        this._super(serverParameters, managementService);

        this.dialog.select(this.view);
        this.view.show(this);
        this._startServiceControllers();
    },

    /** @override */
    reset: function() {
        this.view.reset();
    },

    /** @override */
    valid: function() {
        return this.view.valid();
    },

    getDescriptor: function() {
        this.view.apply();
        return this.descriptor;
    },

    onCompleted: function(descriptor, controller) {
        this.descriptor = descriptor;
    },

    onBroken: function(controller) {
        //ok
    },

    _startServiceControllers: function() {
        for (var i = 0; i < this.controllers.length; i++) {
            this.controllers[i].start(this, this.descriptor, this.serverParameters, this.managementService);
        }
    }
});

/** @class */
jscape.DomainDatastoreServiceView = Class.extend(
    /** @lends {jscape.DomainDatastoreServiceView.prototype} */
{
    views: null,

    /** @constructor */
    initialize: function(views, resources) {
        this.views = views || [];
        this.resources = resources;

        this.fieldset = $("#add-domain-step3");
        this.type = $("#datastore-service");

        this._init();
    },

    asElement: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener;
        this._initState();
    },

    reset: function() {
        this.type.combobox("setValue", this.type.combobox("getData")[0].value);
    },

    valid: function() {
        return this._getSelectedView().valid();
    },

    apply: function() {
        this._getSelectedView().applyChanges();
    },

    _init: function() {
        this.type.combobox({onSelect: $.proxy(this._onChange$Type, this), editable: false});
    },

    _initState: function() {
        this._adjustServiceType();
    },

    _onChange$Type: function() {
        this._adjustServiceType();
    },

    _adjustServiceType: function() {
        var selected = this.type.combobox("getValue");
        var types = this.type.combobox("getData");

        for (var i = 0; i < types.length; i++) {
            var type = types[i].value;
            var view = this._serviceViewFor(type);
            if (type == selected) {
                view.show();
            } else {
                view.hide();
            }
        }
    },

    _getSelectedView: function() {
        return this._serviceViewFor(this.type.combobox("getValue"));
    },

    _serviceViewFor: function(type) {
        return this.views[type];
    }
});

/** @class */
jscape.DomainLogServiceController = jscape.NewDomainModule.extend(
    /** @lends {jscape.DomainLogServiceController.prototype} */
{
    controllers: [],

    /** @constructor */
    initialize: function(dialog, resources) {
        this._super(dialog);

        var fileLogView = new jscape.FileLogServiceView(resources);
        var dbLogView = new jscape.DbLogServiceView(resources);
        this.controllers = [
                new jscape.FileLogServiceController(fileLogView),
                new jscape.DbLogServiceController(dbLogView)
        ];

        this.view = new jscape.DomainLogServiceView([fileLogView, dbLogView], resources);

        this.dialog.addView(this.view);
    },

    /** @override */
    start: function(serverParameters, managementService) {
        this._super(serverParameters, managementService);

        this.dialog.select(this.view);
        this.view.show(this);

        this._startServiceControllers();
    },

    /** @override */
    reset: function() {
        this.view.reset();
    },

    /** @override */
    valid: function() {
        return this.view.valid();
    },

    getDescriptor: function() {
        this.view.apply();
        return this.descriptor;
    },

    onCompleted: function(descriptor, controller) {
        this.descriptor = descriptor;
    },

    onBroken: function(controller) {
        //ok
    },

    _startServiceControllers: function() {
        for (var i = 0; i < this.controllers.length; i++) {
            this.controllers[i].start(this, this.descriptor, this.serverParameters, this.managementService);
        }
    }
});

/** @class */
jscape.DomainLogServiceView = Class.extend(
    /** @lends {jscape.DomainDatastoreServiceView.prototype} */
{
    views: [],

    /** @constructor */
    initialize: function(views, resources) {
        this.views = views || [];
        this.resources = resources;

        this.fieldset = $("#add-domain-step4");
        this.type = $("#log-service");

        this._init();
    },

    asElement: function() {
        return this.fieldset;
    },

    show: function(listener) {
        this.listener = listener;
        this._initState();
    },

    reset: function() {
        this.type.combobox("setValue", this.type.combobox("getData")[0].value);
    },

    valid: function() {
        return this._getSelectedView().valid();
    },

    apply: function() {
        this._getSelectedView().applyChanges();
    },

    _init: function() {
        this.type.combobox({onSelect: $.proxy(this._onChange$Type, this), editable: false});
    },

    _initState: function() {
        this._adjustServiceType();
    },

    _onChange$Type: function() {
        this._adjustServiceType();
    },

    _adjustServiceType: function() {
        var selected = this.type.combobox("getValue");
        var types = this.type.combobox("getData");

        for (var i = 0; i < types.length; i++) {
            var type = types[i].value;
            var view = this._serviceViewFor(type);
            if (type == selected) {
                view.show();
            } else {
                view.hide();
            }
        }
    },

    _getSelectedView: function() {
        return this._serviceViewFor(this.type.combobox("getValue"));
    },

    _serviceViewFor: function(type) {
        return this.views[type];
    }
});