var jscape = jscape || {};
jscape.settings = jscape.settings || {};

/** @class */
jscape.JmxServiceConfiguration = Class.extend(
    /** @lends {jscape.JmxServiceConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(enabled, host, serverPort, registryPort) {
        this.enabled = enabled || false;
        this.host = host;
        this.serverPort = serverPort;
        this.registryPort = registryPort;
    },

    toJSON: function() {
        return {
            enabled: this.enabled,
            host: this.host,
            port: this.registryPort,
            serverPort: this.serverPort
        };
    }
});
jscape.JmxServiceConfiguration.fromJSON = function(data) {
    return new jscape.JmxServiceConfiguration(data.enabled,
                                              data.host,
                                              data.serverPort,
                                              data.port);
};

/** @class */
jscape.settings.JmxController = jscape.settings.SettingsModule.extend(
    /** @lends {jscape.settings.JmxController.prototype} */
{
    /** @const */
    URL_TEMPLATE: "service:jmx:rmi://{host}:{serverPort}/jndi/rmi://{host}:{registryPort}/server",

    view: null,
    config: null,

    /** @constructor */
    initialize: function(resources) {
        this._super(resources);

        this.view = new jscape.settings.JmxView(resources);

        this._initValidators();
    },

    submitChanges: function() {
        var config = this._createConfiguration();
        this.serverSettings.setJmxServiceConfiguration(config);
        this.listener.onChangesSubmitted(this);
    },

    onChange: function() {
        this._adjustJmxUrl();
    },

    _adjustJmxUrl: function() {
        this.view.setUrl(this._createJmxUrl());
    },

    _createJmxUrl: function() {
        var config = this._createConfiguration();
        return this.URL_TEMPLATE.supplant(config);
    },

    _initConfiguration: function() {
        this.config = this.serverSettings.getJmxServiceConfiguration();
    },

    _createConfiguration: function() {
        return new jscape.JmxServiceConfiguration(this.view.getEnabled(),
                                                  this.view.getHost(),
                                                  this.view.getServerPort(),
                                                  this.view.getRegistryPort());
    },

    _setupView: function() {
        var hosts = this.serverSettings.getServerParameters().getInetAddresses();
        var config = this.config;
        this.view.show(this);
        this.view.setHosts(hosts);
        this.view.setHost(config.host);
        this.view.setServerPort(config.serverPort);
        this.view.setRegistryPort(config.registryPort);
        this.view.setEnabled(config.enabled);
    },

    _initValidators: function() {
        $.extend($.fn.validatebox.defaults.rules, {
            'jmx_ports': {
                validator: $.proxy(this._validatorPorts, this)
            }
        });
    },

    _validatorPorts: function() {
        return this.view.getServerPort() != this.view.getRegistryPort();
    }
});

/** @class */
jscape.settings.JmxView = Class.extend(
    /** @lends {jscape.settings.JmxView.prototype} */
{
    /** @const */
    DEFAULT_HOST: "0.0.0.0",
    /** @const */
    DEFAULT_SERVER_PORT: 30051,
    /** @const */
    DEFAULT_REGISTRY_PORT: 30050,

    resources: null,
    listener: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset#ssjmx-fields");
        this.enabled = $("#ssjmx-enable");
        this.host = $("#ssjmx-host");
        this.serverPort = $("#ssjmx-serverport");
        this.registryPort = $("#ssjmx-registryport");
        this.jmxUrl = $("#ssjmx-url");

        this._initComponents();
        this._initListeners();
    },

    show: function(listener) {
        this.listener = listener || jscape.settings.JmxView.LISTENER;
        this._initState();
    },

    getEnabled: function() {
        return this.enabled.prop("checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", value);
        this._adjustFieldState();
    },

    getHost: function() {
        return this.host.combobox("getValue");
    },

    setHost: function(value) {
        value = value || this.DEFAULT_HOST;
        this.host.combobox("setValue", value);
        this.host.combobox("select", value);
    },

    setHosts: function(values) {
        var data = [];
        for (var i = 0; i < values.length; i++) {
            var value = values[i];
            data.push({host: value, value: value});
        }
        this.host.combobox("loadData",data);
    },

    getServerPort: function() {
        return this.serverPort.numberspinner("getValue");
    },

    setServerPort: function(value) {
        this.serverPort.numberspinner("setValue", value || this.DEFAULT_SERVER_PORT);
    },

    getRegistryPort: function() {
        return this.registryPort.numberspinner("getValue");
    },

    setRegistryPort: function(value) {
        this.registryPort.numberspinner("setValue", value || this.DEFAULT_REGISTRY_PORT);
    },

    setUrl: function(url) {
        this.jmxUrl.text(url);
    },

    _initComponents: function() {
        var handler = $.proxy(this._onChange, this);
        this.host.combobox({
            valueField: "host",
            textField: "value",
            onChange: handler,
            disabled: true,
            required: true,
            editable: false,
            missingMessage: this.resources.getString("JMX_ERROR_BAD_HOST")
        });
        this.serverPort.numberspinner({min: 1, max: 65535, disabled: true, onChange: handler, required: true, validType: "jmx_ports", invalidMessage: this.resources.getString("JMX_ERROR_BAD_PORTS")});
        this.registryPort.numberspinner({min: 1, max: 65535, disabled: true, onChange: handler, required: true, validType: "jmx_ports", invalidMessage: this.resources.getString("JMX_ERROR_BAD_PORTS")});
    },

    _initListeners: function() {
        this.enabled.click($.proxy(this._onClick$Enabled, this));
    },

    _initState: function() {
        this._adjustFieldState();
    },

    _onClick$Enabled: function() {
        this._adjustFieldState();
        this.listener.onChange(this);
    },

    _onChange: function() {
        this.listener.onChange(this);
    },

    _adjustFieldState: function() {
        var enabled = this.enabled.prop("checked") ? "enable" : "disable";
        this.host.combobox(enabled).combobox(enabled ? "enableValidation" : "disableValidation");
        this.serverPort.numberspinner(enabled).numberspinner(enabled ? "enableValidation" : "disableValidation");
        this.registryPort.numberspinner(enabled).numberspinner(enabled ? "enableValidation" : "disableValidation");

        this.fieldset.form("validate");
    }
});
/** @const */
jscape.settings.JmxView.LISTENER = {
    onChange: $.noop()
};