/**
 * JSCAPE namespace.
 */
var jscape = jscape || {};

/**Class inheritance */
(function() {
    var fnTest = /xyz/.test(function() {xyz;}) ? /\b_super\b/ : /.*/;
    var initializing = false;
    this.Class = function() {};
    Class.extend = function(prop) {
        var _super = this.prototype;
        initializing = true;
        var prototype = new this();
        initializing = false;

        for (var name in prop) {
            var overwriting = typeof prop[name] == "function"
                              && typeof _super[name] == "function"
                              && fnTest.test(prop[name]);
            prototype[name] = overwriting
                              ? (function(name, fn) {
                                    return function() {
                                        var ref = this._super;
                                        this._super = _super[name];
                                        var result = fn.apply(this, arguments);
                                        this._super = ref;
                                        return result;
                                    }
                                })(name, prop[name])
                              : prop[name];
        }
        function Class() {
            if (!initializing && this.initialize) {
                this.initialize.apply(this, arguments);
            }
        }
        Class.prototype = prototype;
        Class.prototype.constructor = Class;
        Class.extend = arguments.callee;
        return Class;
    };
})();

/**
 * Extends the embedded String type with new functionality.
 * Verifies if current string ends with specified suffix.
 *
 * @param {String} suffix target suffix
 *
 * @return true if current string ends with specified suffix; false otherwise
 *
 * @author Alex Usun
 */
String.prototype.endsWith = function(suffix) {
    var offset = this.length - suffix.length;

    if (offset < 0) {
        return false;
    }

    for (var i = offset; i < this.length; ++i) {
        if (this.charAt(i) != suffix.charAt(i - offset)) {
            return false;
        }
    }

    return true;
};

/**
 * Extends the embedded String type with new functionality.
 * Trims leading and trailing spaces.
 *
 * @return trimmed string
 *
 * @author Alex Usun
 */
String.prototype.trim = function() {
    return this.replace(/(^\s+)|(\s+$)/g, "");
};


/**
 * Replaces {name} variables with specified substitution.
 *
 * @param substitution substitution parameters
 *
 * @author Alex Usun
 */
String.prototype.supplant = function(substitution) {
    return this.replace(/\{([^{}]*)\}/g,
                        function (value, variable) {
                            var replacement = substitution[variable];
                            return typeof replacement === "string" || typeof replacement === "number" ? replacement : value;
                        }
            );
};


/**
 * Returns Error object message.
 *
 * @param {Error} error an error instance
 *
 * @author Alex Usun
 */
jscape.errorMessageFor = function(error) {
    if (error.toString && typeof error.toString === "function") {
        return error.toString();
    } else if (error.message && error.message.length) {
        return error.message;
    } else if (error.description && error.description.length) {
        return error.description;
    }

    return "Unknown error: " + (typeof error);
};


/**
 * A string resource.
 *
 * @param {Object} strings the strings map
 */
jscape.Resources = function(strings) {
    this.strings = strings ? strings : {};
};


jscape.Resources.prototype = {

    /**
     * Returns the key value with optional substitution.
     *
     * @param {String} key the key
     * @param {Object} data the data to substitute
     */
    getString: function(key, data) {
        if (!this.strings[key]) {
            return key;
        }

        var value = this.strings[key];

        return data ? value.supplant(data) : value;
    },

    contains: function(key) {
        return this.strings[key];
    }
};


/**
 * A timer.
 *
 * @param {Function} task a task
 * @param {Number} millis period millis
 * @param {Boolean} repeatable a repeatable task flag
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.Timer = function(task, millis, repeatable) {
    this.task = task;
    this.millis = millis;
    this.repeatable = repeatable;
    this.id = null;
};

jscape.Timer.prototype = {

    /**
     * Starts the timer.
     */
    start: function() {
        if (this.repeatable) {
            this.id = setInterval(this.task, this.millis);
        } else {
            this.id = setTimeout(this.task, this.millis);
        }
    },

    /**
     * Cancels the timer.
     */
    cancel: function() {
        if (!this.id) {
            return;
        }

        if (this.repeatable) {
            clearInterval(this.id);
        } else {
            clearTimeout(this.id);
        }

        this.id = null;
    }
};


/**
 * A time interval.
 *
 * @param {Number} value the time interval in millis
 */
jscape.TimeInterval = function(value) {
    var seconds = Math.round(value / 1000);
    this.hours = this._adjustTimeValue(Math.floor(seconds / 3600));
    this.minutes = this._adjustTimeValue(Math.floor(Math.floor((seconds % 3600)) / 60));
    this.seconds = this._adjustTimeValue(Math.floor(Math.floor((seconds % 3600)) % 60));
};

jscape.TimeInterval.prototype = {

    _adjustTimeValue: function(value) {
        return (value >= 10 ? "" : "0") + value;
    }

};

/** @class */
jscape.ModalDialog = Class.extend(
    /** @lends {jscape.ModalDialog.prototype} */
{
    dialog: null,
    listener: null,

    /**
     * @param {String} id a div id
     * @constructor
     */
    initialize: function(id) {
        this.dialog = this._createDialog(id);
    },

    /**
     * Sets dialog title.
     *
     * @param {String} value a dialog title
     */
    setTitle: function(value) {
        this.dialog.dialog("setTitle", value);
    },

    /**
     * Sets dialog size.
     *
     * @param {Number} width a dialog width
     * @param {Number} height a dialog height
     */
    setSize: function(width, height) {
        this.dialog.dialog("resize", {width: width || "auto", height: height || "auto"});
    },

    /**
     * Add dialog CSS class.
     *
     * @param {String} cssClass dialog class
     */
    addClass: function(cssClass) {
        this.dialog.dialog({bodyCls: cssClass});
    },

    /**
     * Shows dialog.
     *
     * @param listener a dialog listener
     */
    show: function(listener) {
        this.listener = listener || jscape.ModalDialog.LISTENER;
        this.dialog.dialog("center");
        this.dialog.dialog("open");
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        if (!this._closed()) {
            this.dialog.dialog("close");
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _closed: function() {
        return !!this.dialog.dialog("options").closed;
    },

    _createDialog: function(id, dialogClass) {
        var div = $(document.createElement("div"));
        div.attr("id", id);
        return div.dialog({
            buttons: this._buttonsBar(),
            onClose: $.proxy(this._handleClose, this),
            bodyCls: dialogClass,
            closable: true,
            resizable: false,
            closed: true,
            modal: true
        });
    },

    _getContentPane: function() {
        return this.dialog.dialog("body").children().first();
    },

    _setContentPane: function(value) {
        this.dialog.dialog({content: value});
    },

    _buttonsBar: function() {
        return [];
    },

    _button: function(text, fn) {
        return {text: text, handler: $.proxy(fn, this)};
    },

    _handleClose: function() {
        this.listener.onCancel(this);
    }
});
/** @const */
jscape.ModalDialog.LISTENER = {
    onSubmit: function(dialog) {},
    onCancel: function(dialog) {}
};

/** @class */
jscape.MessageDialog = jscape.ModalDialog.extend(
    /** @lends {jscape.MessageDialog.prototype} */
{
    /** @const */
    DEFAULT_WIDTH: 400,

    okButtonText: null,

    /**
     * @param {String} dialogDivId a div id
     * @param {String} okButtonText OK button text
     * @constructor
     */
    initialize: function(dialogDivId, okButtonText) {
        this.okButtonText = okButtonText || "Ok";

        this.dialog = this._createDialog(dialogDivId);
        this.setSize(this.DEFAULT_WIDTH, "auto");
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessage: function(value) {
        this._setContentPane(this._template(value));
    },

    /** @override */
    _buttonsBar: function() {
        return [this._button(this.okButtonText, this._handleSubmit)];
    },

    _handleSubmit: function() {
        this.listener.onSubmit(this);
    },

    _template: function(value) {
        return '<div class="dialog-message">' + value + "</div>";
    }
});

/** @class */
jscape.DetailedMessageDialog = jscape.ModalDialog.extend(
    /** @lends {jscape.DetailedMessageDialog.prototype} */
{
    /** @const */
    DEFAULT_WIDTH: 400,

    okButtonText: null,
    detailsButtonText: null,
    messageText: "",
    detailsText: "",

    /**
     * @param {String} dialogDivId a div id
     * @param {String} okButtonText an OK button text
     * @param {String} detailsButtonText a details button text
     * @constructor
     */
    initialize: function(dialogDivId, okButtonText, detailsButtonText) {
        this.okButtonText = okButtonText || "Ok";
        this.detailsButtonText = detailsButtonText || "Details";

        this.dialog = this._createDialog(dialogDivId);
        this.setSize(this.DEFAULT_WIDTH, "auto");
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessage: function(value) {
        this.messageText = value;
        this._updateContent();
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog details
     */
    setDetails: function(value) {
        this.detailsText = value;
        this._updateContent();
    },

    /** @override */
    _buttonsBar: function() {
        return [
            this._button(this.okButtonText, this._handleSubmit),
            this._button(this.detailsButtonText, this._handleDetailsPerformed)
        ];
    },

    _handleSubmit: function() {
        this.listener.onSubmit(this);
    },

    _handleDetailsPerformed: function() {
        this._detailsContainer().toggle();
    },

    _detailsContainer: function() {
        return this.dialog.dialog("body").find(".dialog-details");
    },

    _updateContent: function() {
        this._setContentPane(this._template(this.messageText || "", this.detailsText || ""));
    },

    _template: function(message, details) {
        return '<div class="dialog-message">{message}</div><div class="dialog-details"">{details}</div>'
                .supplant({message: message, details: details});
    }
});


/** @class */
jscape.ConfirmationDialog = jscape.MessageDialog.extend(
    /** @lends {jscape.ConfirmationDialog.prototype} */
{
    /** @const */
    DEFAULT_WIDTH: 400,

    confirmButtonText: null,
    cancelButtonText: null,

    /**
     * @param {String} dialogDivId a div id
     * @param {String} confirmButtonText a confirm button text
     * @param {String} cancelButtonText a cancel button text
     * @constructor
     */
    initialize: function(dialogDivId, confirmButtonText, cancelButtonText) {
        this.confirmButtonText = confirmButtonText || "Ok";
        this.cancelButtonText = cancelButtonText || "Cancel";

        this.dialog = this._createDialog(dialogDivId);
        this.setSize(this.DEFAULT_WIDTH, "auto");
    },

    /** @override */
    _buttonsBar: function() {
        return [
            this._button(this.confirmButtonText, this._handleConfirm),
            this._button(this.cancelButtonText, this._handleCancel)
        ];
    },

    _handleConfirm: function() {
        this.listener.onSubmit(this);
    },

    _handleCancel: function() {
        this.listener.onCancel(this);
    }
});

/** @class */
jscape.ProgressDialog = jscape.ModalDialog.extend(
    /** @lends {jscape.ProgressDialog.prototype} */
{
    /** @const */
    DEFAULT_HEIGHT: 80,
    /** @const */
    DEFAULT_WIDTH: 500,

    container: null,

    /**
     * @param {String} dialogDivId a dialog div id
     * @constructor
     */
    initialize: function(dialogDivId) {
        this._super(dialogDivId);

        this._setContentPane("<div></div>");
        this.container = this._getContentPane();
        this.container.progressbar({value: 0});

        this.setSize(this.DEFAULT_WIDTH, this.DEFAULT_HEIGHT);
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    progress: function(percents) {
        this.container.progressbar("setValue", percents);
    }
});

/** @class */
jscape.PromptDialog = jscape.ModalDialog.extend(
    /** @lends {jscape.PromptDialog.prototype} */
{
    /** @const */
    DEFAULT_OPTIONS: {width: 530, resizable: false, closed: true, closable: true, modal: true},

    resources: null,

    /** @constructor */
    initialize: function(dialog, resources) {
        this.resources = resources;
        this.dialog = this._createDialog(dialog);
    },

    /** @override */
    show: function(listener) {
        this._super(listener);
        this.dialog.find("input:first").focus();
    },

    _valid: function() {
        return true;
    },

    _handleSubmit: function() {
        if (this._valid()) {
            this.listener.onSubmit(this);
        }
    },

    _handleCancel: function() {
        this.listener.onCancel(this);
    },

    _handleClose: function() {
        this.listener.onCancel(this);
    },

    /** @override */
    _createDialog: function(dialog) {
        dialog.dialog($.extend({}, this.DEFAULT_OPTIONS, {
            onClose: $.proxy(this._handleClose, this),
            buttons: [
                {text: this.resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._handleSubmit, this)},
                {text: this.resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._handleCancel, this)}
            ]
        }));
        dialog.dialog("body").css({display: "block"});
        return dialog;
    }
});

/** @class */
jscape.ConfirmDialog = {};
jscape.ConfirmDialog.confirm = function(title, message) {
    var deferred = $.Deferred();
    $.messager.confirm(title, message, function(confirmed) {
        if (confirmed) {
            deferred.resolve(confirmed);
        } else {
            deferred.reject();
        }
    });
    return deferred.promise();
};

/** @class */
jscape.WizardDialog = jscape.PromptDialog.extend(
    /** @lends {jscape.WizardDialog.prototype} */
{
    /** @const */
    DEFAULT_OPTIONS: {width: 650, height: 400, resizable: true, closed: true, closable: true, modal: true},
    /** @const */
    BUTTON_IDS: {PREV: "wizard-prevbtn", NEXT: "wizard-nextbtn", OK: "wizard-okbtn", CANCEL: "wizard-cancelbtn"},

    currentStep: -1,
    views: [],

    /** @constructor */
    initialize: function(dialog, resources) {
        this._super(dialog, resources);

        this._initComponents();
    },

    /** @override */
    show: function(listener) {
        this.currentStep = 0;
        this.select(this.views[this.currentStep]);
        this._super(listener);
    },

    select: function(view) {
        for (var i = 0; i < this.views.length; i++) {
            var v = this.views[i];
            this._setViewVisible(v, v === view);
        }
        this._adjustButtons();
    },

    _initComponents: function() {
        var views = $("fieldset", this.dialog).siblings();
        for (var i = 0; i < views.length; i++) {
            var view = views[i];
            this._setViewVisible(view, false);
            this.views.push(view);
        }
    },

    _setViewVisible: function(view, visible) {
        if (visible) {
            $(view).show();
        } else {
            $(view).hide();
        }
    },

    /** @override */
    _createDialog: function(dialog) {
        dialog.dialog($.extend({}, this.DEFAULT_OPTIONS, {
            onClose: $.proxy(this._handleClose, this),
            buttons: [{
                id: this.BUTTON_IDS.PREV,
                text: this.resources.getString("DIALOG_BUTTON_PREVIOUS") || "Previous",
                handler: $.proxy(this._handlePrevStep, this)
            }, {
                id: this.BUTTON_IDS.NEXT,
                text: this.resources.getString("DIALOG_BUTTON_NEXT") || "Next",
                handler: $.proxy(this._handleNextStep, this)
            }, {
                id: this.BUTTON_IDS.OK,
                text: this.resources.getString("DIALOG_BUTTON_OK") || "Ok",
                iconCls: "icon-ok",
                handler: $.proxy(this._handleSubmit, this)
            }, {
                id: this.BUTTON_IDS.CANCEL,
                text: this.resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel",
                iconCls: "icon-cancel",
                handler: $.proxy(this._handleCancel, this)
            }]
        }));
        dialog.dialog("body").css({display: "block"});
        return dialog;
    },

    _handlePrevStep: function() {
        if (this.listener.onPrevious(this)) {
            this.select(this.views[--this.currentStep]);
        }
    },

    _handleNextStep: function() {
        if (this.listener.onNext(this)) {
            this.select(this.views[++this.currentStep]);
        }
    },

    _adjustButtons: function() {
        var prevButton = this.dialog.find("#" + this.BUTTON_IDS.PREV);
        var nextButton = this.dialog.find("#" + this.BUTTON_IDS.NEXT);
        var okButton = this.dialog.find("#" + this.BUTTON_IDS.OK);

        if (this.currentStep > 0) {
            prevButton.show();
            if (this.currentStep == this.views.length - 1) {
                nextButton.hide();
                okButton.show();
            } else {
                nextButton.show();
                okButton.hide();
            }
        } else {
            prevButton.hide();
            nextButton.show();
            okButton.hide();
        }
    }
});
jscape.WizardDialog.LISTENER = {
    onPrevious: function() {},
    onNext: function() {},
    onSubmit: function() {},
    onCancel: function() {}
};

/**
 * A time message progress dialog.
 * @class
 */
jscape.TimeMessageDialog = jscape.ModalDialog.extend(
    /** @lends {jscape.TimeMessageDialog.prototype} */
{
    /** @const */
    DEFAULT_HEIGHT: 300,
    /** @const */
    UPDATE_MILLIS: 1000,
    /** @const */
    DEFAULT_WIDTH: 400,

    startTime: 0,
    percents: "",
    messageTemplate: "",
    timer: null,

    /**
     * @param {String} dialogDivId a dialog div id
     * @constructor
     */
    initialize: function(dialogDivId, dialogClass) {
        this._super(dialogDivId, dialogClass);

        //this.setSize(this.DEFAULT_WIDTH, this.DEFAULT_HEIGHT);
        this.timer = new jscape.Timer($.proxy(this._update, this), this.UPDATE_MILLIS, true);
    },

    /**
     * Shows dialog.
     */
    show: function() {
        this.startTime = new Date().getTime();
        this.updateProgress(0);
        this.timer.start();

        this._super();
    },

    /**
     * Sets dialog message.
     *
     * @param {String} value a dialog message
     */
    setMessageTemplate: function(value) {
        this.messageTemplate = value;
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        this.percents = percents;
        this._update();
    },

    _onClose: function() {
        this.timer.cancel();
        this._super();
    },

    _update: function() {
        var substitution = this._getTakenTime();
        substitution["percents"] = this.percents;
        var message = this.messageTemplate.supplant(substitution);
        this.dialog.html(message);
    },

    _getTakenTime: function() {
        var view = this;
        var takenMillis = Math.round(new Date().getTime() - this.startTime);
        return new jscape.TimeInterval(takenMillis);
    }
});


/**
 * A wait cursor dialog.
 *
 * @param {String} template a status line message template
 *
 * @constructor
 *
 * @author Alex Usun
 */
jscape.WaitCursorDialog = function(template) {
    this.statusTemplate = template ? template : this.DEFAULT_TEMPLATE;
};

jscape.WaitCursorDialog.prototype = {

    DEFAULT_TEMPLATE: "{percents}%",

    /**
     * Shows dialog.
     */
    show: function() {
        document.body.style.cursor = "wait";
    },

    /**
     * Hides dialog.
     */
    hide: function() {
        document.body.style.cursor = "default";
    },

    /**
     * Updates progress.
     *
     * @param {Number} percents completeness percents
     */
    updateProgress: function(percents) {
        window.status = this.statusTemplate.supplant({percents: percents});
    }
};


/**
 * AJAX/REST operation error data.
 * @class
 */
jscape.ErrorData = Class.extend(
    /** @lends {jscape.ErrorData.prototype} */
{
    /**
     * @param {String} httpCode an HTTP error code
     * @param {String} httpText a HTTP error text
     * @param {String} responseText server response text
     * @param {Object} status status
     * @param {Object} error error
     * @constructor
     */
    initialize: function(httpCode, httpText, responseText, status, error) {
        this.httpCode = httpCode;
        this.httpText = httpText;
        this.serverText = responseText;

        var data = this._parseResponse(responseText) || {};
        this.serverCode = data.code || "";
        this.serverMessage = data.message || "";
        this.serverDetails = data.details || "";
        this.status = status;
        this.error = error;
    },

    _parseResponse: function(response) {
        try {
            return $.parseJSON(response);
        } catch (e) {
            return {code: "", message: "", details: ""};
        }
    }
});

/** @class */
jscape.RestOperationErrorDialog = jscape.DetailedMessageDialog.extend(
    /** @lends {jscape.RestOperationErrorDialog.prototype} */
{
    resources: null,

    /**
     * @param {String} dialogDivId a div id
     * @param {String} okButtonText an OK button text
     * @param {String} detailsButtonText a details button text
     * @param {String} title the dialog title
     * @param {jscape.Resources} resources a dialog resources
     * @constructor
     */
    initialize: function(dialogDivId, okButtonText, detailsButtonText, title, resources) {
        this._super(dialogDivId, okButtonText, detailsButtonText);

        this.resources = resources;
        this.setTitle(title);
    },

    /**
     * Sets the error.
     *
     * @param {jscape.ErrorData} errorData error data
     */
    setError: function(errorData) {
        this.setMessage(this.resources.contains(errorData.serverCode)
                        ? this.resources.getString(errorData.serverCode)
                        : this.resources.getString("SERVER_ERROR"));
        this.setDetails(this._detailsOf(errorData));
    },

    _detailsOf: function(errorData) {
        return ("<p><strong>HTTP response: </strong>{httpCode} ({httpText})</p>"
                + "<p><strong>Server response: </strong> {serverCode} ({serverMessage})</p>"
                + "<p>{serverDetails}</p>").supplant(errorData);
    }
});

/** @class */
jscape.NullDialog = jscape.ModalDialog.extend({

    /** @constructor */
    initialize: function() {
    },

    /** @override */
    show: function(listener) {
        (listener || jscape.ModalDialog.LISTENER).onCancel(this);
    },

    /** @override */
    hide: function() {
    },

    /** @override */
    setTitle: function() {
    },

    setError: function(data) {
        var message = "{";
        for (var key in data) {
            message += key + ":" + data[key] + ",";
        }
        message += "}";
        window.console && window.console.log("Error: " + message);
    },

    /** @override */
    setSize: function() {
    },

    /** @override */
    _createDialog: function() {
        return null;
    }
});

/** @class */
jscape.AjaxOperation = Class.extend(
    /** @lends {jscape.AjaxOperation.prototype} */
{
    method: "GET",
    contentType: "application/x-www-form-urlencoded;charset=utf-8",
    dataType: null,
    timeoutMillis: 180000,
    listener: null,

    /**
     * @param {Number} timeoutMillis a connection timeout in milliseconds
     * @constructor
     */
    initialize: function(timeoutMillis) {
        this.timeoutMillis = timeoutMillis;
    },

    /**
     * Starts operation.
     *
     * @param listener operation listener
     * @param {String} url an connection URL
     * @param {String} data data to send
     */
    start: function(listener, url, data) {
        this.listener = listener || jscape.AjaxOperation.LISTENER;

        jQuery.ajax({
            type: this.method,
            url: url,
            cache: false,
            timeout: this.timeoutMillis,
            contentType: this.contentType,
            data: data,
            processData: false,
            dataType: this.dataType,

            success: $.proxy(this._handleSuccess, this),
            error: $.proxy(this._handleError, this)
        });
    },

    _handleSuccess: function(data, textStatus, request) {
        var statusCode = request.status;
        if (statusCode === 0) {
            this.listener.onError(this, new jscape.ErrorData("Connection error.", "", "Connection error.", status, null));
        } else {
            this.listener.onComplete(this, data, textStatus, statusCode);
        }
    },

    _handleError: function(request, status, error) {
        this.listener.onError(this, this._getErrorData(request, status, error));
    },

    _getErrorData: function(request, status, error) {
        try {
            return new jscape.ErrorData(request.status,
                                        request.statusText,
                                        request.responseText,
                                        status,
                                        error);
        } catch(e) {
            return new jscape.ErrorData("Browser error", jscape.errorMessageFor(e), "", status, error);
        }
    }
});
/** @const */
jscape.AjaxOperation.LISTENER = {
    onComplete: function(operation, data, status, code) {},
    onError: function(operation, errorData) {}
};

/** @class */
jscape.RestOperation = Class.extend(
    /** @lends {jscape.RestOperation.prototype} */
{
    url: null,
    method: "POST",
    timeout: 0,
    progressDialog: null,
    errorMessageDialog: null,

    /**
     * @param {String} url URL
     * @param {String} method request method
     * @param {Number} timeoutMillis a connection timeout in milliseconds
     * @param {Object} progressDialog a progress dialog
     * @param {Object} errorMessageDialog an error message dialog
     *
     * @constructor
     */
    initialize: function(url, method, timeoutMillis, progressDialog, errorMessageDialog) {
        this.url = url;
        this.method = method;
        this.timeout = timeoutMillis;
        this.progressDialog = progressDialog;
        this.errorMessageDialog = errorMessageDialog;
    },

    start: function(listener, data) {
        this.listener = listener || jscape.RestOperation.LISTENER;
        this.progressDialog.show();

        var operation = new jscape.AjaxOperation(this.timeout);
        operation.method = this.method || "POST";
        operation.contentType = "application/json";
        operation.dataType = "text";

        operation.start({
            onComplete: $.proxy(this._handleComplete, this),
            onError: $.proxy(this._handleError, this)
        }, this.url, data ? JSON.stringify(data) : "");
    },

    _handleComplete: function(operation, response, status, code) {
        this.progressDialog.hide();
        this.listener.onCompleted(this._parseResponse(response, code));
    },

    _handleError: function(operation, errorData) {
        this.progressDialog.hide();
        this.errorMessageDialog.setError(errorData);
        this.errorMessageDialog.show({
            onSubmit: $.proxy(this._handleErrorDialog, this),
            onCancel: $.proxy(this._handleErrorDialog, this)
        });
    },

    _handleErrorDialog: function() {
        this.errorMessageDialog.hide();
        this.listener.onBroken();
    },

    _parseResponse: function(response, statusCode) {
        return statusCode == 204 ? response : $.parseJSON(response);
    }
});
/** @const */
jscape.RestOperation.LISTENER = {
    onCompleted: function(data) {},
    onBroken: function() {}
};


/** @class */
jscape.Hours = Class.extend({
    /** @const */
    SECONDS_PER_MINUTE: 60,
    /** @const */
    MINUTES_PER_HOUR: 60,
    /** @const */
    MILLISECONDS_PER_SECOND: 1000,
    /** @const */
    MILLISECONDS_PER_MINUTE: 1000 * 60,
    /** @const */
    MILLISECONDS_PER_HOUR: 1000 * 60 * 60,

    hours: 0,
    minutes: 0,
    seconds: 0,
    millis: 0,

    /** @lends {jscape.Hours.prototype} */
    initialize: function(value) {
        this.hours = value / this.MILLISECONDS_PER_HOUR | 0;
        value %= this.MILLISECONDS_PER_HOUR;
        this.minutes = value / this.MILLISECONDS_PER_MINUTE | 0;
        value %= this.MILLISECONDS_PER_MINUTE;
        this.seconds = value / this.MILLISECONDS_PER_SECOND | 0;
        this.millis = value % this.MILLISECONDS_PER_SECOND | 0;
    }
});

/** @class */
jscape.SizeFormat = Class.extend(
    /** @lends {jscape.SizeFormat.prototype} */
{
    /** @const */
    /** @const */
    SIZE: {
        BYTE: {factor: 1, unit: "byte(s)"},
        KB: {factor: 1024, unit: "KB"},
        MB: {factor: 1024 * 1024, unit: "MB"},
        GB: {factor: 1024 * 1024 * 1024, unit: "GB"},
        TB: {factor: 1024 * 1024 * 1024 * 1024, unit: "TB"}
    },

    format: function(value) {
        var size;
        if (value < this.SIZE.KB.factor) {
            size = this.SIZE.BYTE;
        } else if (this.SIZE.KB.factor <= value && value < this.SIZE.MB.factor) {
            size = this.SIZE.KB;
        } else if (this.SIZE.MB.factor <= value && value < this.SIZE.GB.factor) {
            size = this.SIZE.MB;
        } else if (this.SIZE.GB.factor <= value && value < this.SIZE.TB.factor) {
            size = this.SIZE.GB;
        } else {
            size = this.SIZE.TB;
        }
        var digits = value == 0 ? 0 : (size == this.SIZE.BYTE ? 1 : 2);
        return (value / size.factor).toFixed(digits) + " " + size.unit;
    }
});

/** @class */
jscape.DateFormat = Class.extend(
    /** @lends {jscape.DateFormat.prototype} */
{
    /** @constructor */
    initialize: function(pattern) {
        this.pattern = pattern || "{hh}:{mm}:{ss} {MM}-{dd}-{yy}";
    },

    format: function(date) {
        var year = date.getFullYear();
        var month = date.getMonth() + 1;
        var day = date.getDate();
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();

        return this.pattern.supplant({
            d: day, dd: this._leadingZero(day),
            M: month, MM: this._leadingZero(month),
            y: year, yy: this._leadingZero(year),
            h: hours, hh: this._leadingZero(hours),
            m: minutes, mm: this._leadingZero(minutes),
            s: seconds, ss: this._leadingZero(seconds)
        });
    },
    _leadingZero: function(value) {
        return (value < 10 ? "0" : "") + value;
    }
});